package org.sinev.igor.applications;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;

import org.sinev.igor.ps.PSParseException;
import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.DocumentFactory;
import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.IPSEInteger;
import org.sinev.igor.ps.model.IPSEReal;
import org.sinev.igor.ps.model.IStackObject;
import org.sinev.igor.ps.model.impl.ElementFactory;
import org.sinev.igor.ps.operatordescription.OperationPerformer;
import org.sinev.igor.ps.operatordescription.OperatorDescription;
import org.sinev.igor.ps.operatordescription.OperatorNamingMap;
import org.sinev.igor.ps.reader.PSReader;

/**
 * Demonstrates execution of add operation.
 * Calculates all values that are left after execution on the stack.
 * {@link org.sinev.igor.ps.model.DocumentFactory#DEFAULT_NAMING_MAP}
 * 
 * @author Igor Sinev
 */
public class AdditionCalculator {
	public static void main(String[] args) {
		if (args.length != 1) {
			System.err.println("usage:\n  java " + AdditionPerformer.class.getName() + " <input file name>");
			return;
		}
		
		IDocument psDocument;
		try {			
			psDocument = DocumentFactory.getDocumentFromReader(new PSReader(new FileReader(args[0])));			
			OperatorNamingMap m = DocumentFactory.getNamingMap();
			OperatorDescription d = m.getDescription("add");
			d.setPerformer(new AdditionPerformer());
			for (IStackObject o : psDocument.getOutgoingValues())
				System.out.println(o.getElement());						
		} catch (FileNotFoundException e) {			
			e.printStackTrace();
		} catch (PSParseException e) {
			e.printStackTrace();
		}
	}
	public static class AdditionPerformer implements OperationPerformer {
		public List<IElement> calculate(ArrayList<IElement> args, IDocument doc) {
			IElement arg1 = args.get(0);
			IElement arg2 = args.get(1);
			IElement r;
			if (arg1.getType().combinateWith(arg2.getType()) == ElementType.INT) {
				r = ElementFactory.createIntegerElement(
						((IPSEInteger) arg1).getValue() + ((IPSEInteger) arg2).getValue(),
						doc
					);
			}
			else {					
				double s = 0;
				if (arg1 instanceof IPSEInteger)
					s += ((IPSEInteger) arg1).getValue();
				else
					s += ((IPSEReal) arg1).getValue();
				
				if (arg2 instanceof IPSEInteger)
					s += ((IPSEInteger) arg2).getValue();
				else
					s += ((IPSEReal) arg2).getValue();
				
				r = ElementFactory.createRealElement(s, doc);
			}
			List<IElement> res = new ArrayList<IElement>();
			res.add(r);
			return res;
		}
	}
	
}
