package org.sinev.igor.ps.model;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import org.sinev.igor.ps.PSParseException;
import org.sinev.igor.ps.model.impl.Document;
import org.sinev.igor.ps.model.impl.Element;
import org.sinev.igor.ps.operatordescription.OperatorNamingMap;
import org.xml.sax.SAXException;

public class DocumentFactory {

	private static final String DEFAULT_NAMING_MAP = "org\\sinev\\igor\\ps\\tests\\operators.xml";
	private static OperatorNamingMap ourOperatorNamingMap = null;

	/**
	 * Creates a PS document and then uses specified 
	 * reader to get it's contents.
	 * 
	 * @param r reader go get document from.
	 * 
	 * @return constructed document.
	 * 
	 * @throws PSParseException
	 */
	public static Document getDocumentFromReader(IReader r) throws PSParseException {
		Document d = new Document();
		if (ourOperatorNamingMap == null) {			
			try {
				setNamingMap(OperatorNamingMap.getFromXML(new File(DEFAULT_NAMING_MAP)));
			} catch (ParserConfigurationException e) {
				throw new PSParseException("Exception occured while attempting to use default operator naming map", e);
			} catch (SAXException e) {
				throw new PSParseException("Exception occured while attempting to use default operator naming map", e);
			} catch (IOException e) {
				throw new PSParseException("Exception occured while attempting to use default operator naming map", e);
			}
		}
		d.setNamingMap(ourOperatorNamingMap);
				
	    for (;;) {
	        Element e = (Element) r.getNextElementFor(d);
	        if (e == null)
	            break;
	        d.addElement(e);
	    }
	    d.buildOperatorLinks();
	    		
		return d;
	}

	/**
	 * Gets a link to an {@link OperatorNamingMap} that is used 
	 * to work with PS documents.
	 *  
	 * @return naming map that is currently used.
	 */
	public static OperatorNamingMap getNamingMap() {
		return ourOperatorNamingMap;
	}

	/**
	 * Sets an {@link OperatorNamingMap} that is used 
	 * to work with PS documents.
	 * 
	 * @param m naming map that should be used.
	 */
	public static void setNamingMap(OperatorNamingMap m) {
		ourOperatorNamingMap = m;		
	}	
}
