package org.sinev.igor.ps.model.impl;

import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.ICalculatedStackObject;
import org.sinev.igor.ps.model.IElement;

/**
 * Piece of data flow that is generated as a result 
 * of some other operation.
 * 
 * @author Igor Sinev 
 */
public class CalculatedStackObject implements ICalculatedStackObject {

	/**
	 * Number of this object in operator returns list.
	 */
	private int myNumber;
	
	/**
	 * Type of element corresponding to this object.
	 */
	private ElementType myType = null;
	
	/**
	 * Functinoal element that produces this object.
	 */
	private FunctionalElement myFunctionalElement;
	
	CalculatedStackObject(FunctionalElement f, int n) {
		myFunctionalElement = f;
		myNumber = n;
	}
	
	CalculatedStackObject(FunctionalElement f, int n, ElementType t) {
		this(f, n);
		myType = t;
	}
	
	/**
	 * Sets special element type for this object.
	 * 
	 * @param t type to set.
	 */
	void setType(ElementType t) {
		myType = t;
	}
	
	public ElementType getType() {
		if (myType == null)			
			myType = myFunctionalElement.getReturnType(myNumber); 
		return myType;
	}
	
	public IElement getElement() {		
		return myFunctionalElement.calculateReturns().get(myNumber);
	}	
	
	public String toString() {
		return myFunctionalElement + ":" + myNumber;
	}	

	/**
	 * Writes debug information concerning structure.
	 * 
	 * @param target buffer to write to.
	 */
	public void callStructureDebugOutput(StringBuffer target) {
		myFunctionalElement.structureDebugOutput(target);
	}
		
	public FunctionalElement getFunctionalElement() {
		return myFunctionalElement;
	}
}
