package org.sinev.igor.ps.model.impl;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Stack;

import org.sinev.igor.collections.ThreadedList;
import org.sinev.igor.ps.PSParseException;
import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.IElementStructure;
import org.sinev.igor.ps.model.IStackObject;

/**
 * Block of elements with internal data flow links.
 * 
 * @author Igor 
 */
public class ElementStructure implements IElementStructure {
	
	/**
	 * Special stack that stores information about all reqests 
	 * 
	 * @author Igor Sinev
	 */
	public static class StorageStack implements Iterable<IStackObject> {
		
		Stack<IStackObject> myContents = new Stack<IStackObject>();
		Stack<ElementType> myDebt = new Stack<ElementType>();
		
		
		/**
		 * Gets an element from stack. If it's already empty, 
		 * information about reqest is stored.
		 * 
		 * @param t expected element type
		 * 
		 * @return read element or null if stack is empty
		 */
		public IStackObject pop(ElementType t) {
			if (myContents.size() > 0)
				return myContents.pop();
			else {			
				myDebt.push(t);
				return null;
			}		
		}

		/**
		 * Adds a new element to the top of stack
		 * 
		 * @param o element to add.
		 * 
		 * @return
		 */
		public void push(IStackObject o) {
			myContents.push(o);
		}

		public Iterator<IStackObject> iterator() {
			return myContents.iterator();
		}

	}

	private ThreadedList myBody = new ThreadedList();
	private ArrayList<IStackObject> myOutgoingValues = new ArrayList<IStackObject>();
	
	/**
	 * List of elements that should be at the top of stack 
	 * at the beginning of procedure processing.
	 */
	protected ArrayList<ElementType> myDebt;
	
	/**
	 * Adds an element to the end of a document.
	 * 
	 * @param e element to add.
	 */
	public void addElement(Element e) {
		myBody.add(e);	
	}
	
	protected void collectOutgoing(StorageStack es) {
		for (IStackObject so : es)
	    	myOutgoingValues.add(so); 
	}
	
		
	public List<IElement> getElementsList() {
		return Collections.unmodifiableList(myBody);		
	}
	
	public List<IStackObject> getOutgoingValues() {
		return Collections.unmodifiableList(myOutgoingValues);
	}
		
	public ThreadedList getBody() {		
		return myBody;
	}

	public void buildOperatorLinks() throws PSParseException  {
		StorageStack es = new StorageStack();
		for (IElement ie : myBody) {
			Element e = (Element) ie;
			if (e.isInterpretable())
	        	e.putToStack(es);
	    }
	    collectOutgoing(es);
	    myDebt = new ArrayList<ElementType>(es.myDebt);
	}
}
