package org.sinev.igor.ps.model.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.IFunctionalElement;
import org.sinev.igor.ps.model.IStackObject;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

/**
 * Element that performs some operations.
 * 
 * @author Igor Sinev
 */
public abstract class FunctionalElement extends Element implements IFunctionalElement {
	
	/**
	 * List containing links to the arguments of this operator.
	 */
	protected ArrayList<IStackObject> myArguments;
	/**
	 * List containing return values of this operator.
	 */
	protected ArrayList<IElement> myReturns;
	/**
	 * Determines if this operator was already calculated.
	 */
	protected boolean myCalculated = false;

	protected FunctionalElement(String s, IDocument d) {
		super(s, d);		
	}
	
	public ElementType getType() {
		throw new IllegalStateException("Objects of type " + this.getClass().getName() + " can't take part in interpretation data flow as operator arguments");
	}

	@Override
	public boolean isInterpretable() {
		return true;
	}	

	/**
	 * Carries out partial interpretation of a single functional element 
	 * of a PS program. 
	 */
	protected void calculate() {
		throw new NotImplementedException();
	}
	
	public List<IElement> calculateReturns() {
		calculate();
		return Collections.unmodifiableList(myReturns);
	}
	
	/**
	 * Gets type of piece of output data produced by this element 
	 * by its number.
	 * 
	 * @param myNumber number of return value
	 * 
	 * @return type of return value
	 */
	public abstract ElementType getReturnType(int myNumber);

}
