package org.sinev.igor.ps.model.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.sinev.igor.ps.elementtype.ElementType;
import org.sinev.igor.ps.model.IDocument;
import org.sinev.igor.ps.model.IElement;
import org.sinev.igor.ps.model.IPSEArrayElement;

/**
 * Represents an array element of some PS document.
 * 
 * @author Igor Sinev
 */
public class PSEArrayElement extends Element implements IPSEArrayElement {
	
	private ArrayList<IElement> myContents;

	protected PSEArrayElement(String representation, IDocument d) {
		super(representation, d);
		throw new UnsupportedOperationException();
	}

	/**
	 * Creates new element.
	 * 
	 * @param contents elements of array
	 * @param representation string representation
	 * @param d document containing the element
	 */
	protected PSEArrayElement(IElement[] contents, String representation, IDocument d) {
		super(representation, d);
		myContents = new ArrayList<IElement>(contents.length);
		for (IElement e : contents)
			myContents.add(e);		
	}

	@Override
	public boolean isInterpretable() {
		return true;
	}

	public ElementType getType() {
		return ElementType.ARRAY;
	}
	
	public List<IElement> getValue() {
		return Collections.unmodifiableList(myContents);	
	}
}
