package org.sinev.igor.ps.reader;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PushbackReader;
import java.io.Reader;
import java.nio.CharBuffer;

import org.sinev.igor.ps.PSParseException;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

/**
 * Buffered reader with pushback that counts read characters.
 * 
 * @author Igor Sinev
 */
public class CountingPushbackReader extends PushbackReader {
	private int myCount = 0;	
	
	@Override
	public int read() throws IOException {
		myCount++;
		return super.read();
	}
	
	/**
	 * Throws exception with specified message if end of stream is reached.
	 * 
	 * @param exceptionMessage message for exception
	 * 
	 * @return read character
	 * 
	 * @throws IOException
	 * @throws PSParseException
	 */
	public char eofSafeRead(String exceptionMessage) throws IOException, PSParseException {
		int i = read();
		if (i == -1)
			throw new PSParseException("Unexpected EOF: " + exceptionMessage);
		else
			return (char) i;
	}
	
	@Override
	public void unread(int c) throws IOException {
		myCount--;
		super.unread(c);		
	}
	
	/**
	 * Creates a new pushback reader from given reader.
	 * At first bufferization is added to the reader.
	 * 
	 * @param r Reader to get data from. 
	 */
	CountingPushbackReader(Reader r) {
		super(new BufferedReader(r));
	}
	
	/**
	 * Gets characters count.
	 * 
	 * @return number of characters that was already read.
	 */
	public int getCount() {
		return myCount;
	}
		
	
	@Override
	public int read(char[] cbuf, int off, int len) throws IOException {
		throw new NotImplementedException();
	}

	@Override
	public void unread(char[] cbuf, int off, int len) throws IOException {
		throw new NotImplementedException();
	}

	@Override
	public void unread(char[] cbuf) throws IOException {
		throw new NotImplementedException();
	}

	@Override
	public int read(char[] cbuf) throws IOException {
		throw new NotImplementedException();
	}

	@Override
	public int read(CharBuffer target) throws IOException {
		throw new NotImplementedException();
	}	
}